# -*- coding: utf-8 -*-
"""
CREATED
    @author: Morten Lybech Thøgersen, 2023-05-22, mlt@emd.dk

PURPOSE:
    An example on howto retrieve  EMD-WRF OD mesoscale results files from through the EMD-API.
    This is a minimal example by use of the python requests package.
    The data downloaded into .mesores file - saved in your temporary folder.

    This file is a part of set of 3 example files each with a specific focus:
       1. Status and overview of user-request and past-calculations (emdwrfod_example_status.py)
       2. Submit a calculation (emdwrfod_example_order_submit.py)
       3. Check status and retrieve data from one or more calculations (emdwrfod_example_order_check.py)

USAGE:
    1. Obtain a API access token from EMD (contact sales@emd.dk)
    2. Purchase some calculation-credits (1 credit is worth 1 month of mesoscale-timeseries for the basic configuration)
    3. Run this script and inspect the results

REFERENCES:
    .mesores documentation with samples:
       https://help.emd.dk/knowledgebase/
       https://www.emd.dk/files/flow/EMD_technote_MESORES_20230426.pdf
    UI: https://api.emd.dk/emd-wrf-od/ui

"""

import os.path
import shutil
import requests
import json
import tabulate
import pandas as pd
import tempfile


if __name__ == '__main__':

    # Input data: URL, Access-Token and Model-Id
    api_url = "http://api.emd.dk/emd-wrf-od"
    user_token = '16d5e6de-74da-4d03-a06c-ee1942417959'  # EMD-API OD TEST USER

    # Create data-frame of all calculations
    response_info = requests.get(api_url + "/info", headers={'X-Auth': user_token}, verify=True)
    info_status = json.loads(response_info.content)
    print("\nACCOUNT INFORMATION - CALCULATIONS")
    print("----------------------------------")
    df_calculations = pd.DataFrame(data=info_status['calculations'])
    if not(df_calculations.empty):
        df_calculations = df_calculations[['datetime', 'status', 'user', 'credits', 'order_id', 'configuration']]
        df_calculations = df_calculations.sort_values('datetime', ascending=False)
    print(tabulate.tabulate(df_calculations, headers=df_calculations.columns, showindex=False))

    # Loop through list of all orders - and download data
    print("\nACCOUNT INFORMATION - STATUS OF ALL PREVIOUS ORDERS AND DOWNLOAD")
    print("----------------------------------------------------------------")
    print("   Results are saved into the local temp-folder: ", tempfile.gettempdir())
    for idx, row in df_calculations.iterrows():
        orderid = row["order_id"]
        try:
            response_status = requests.get(api_url + "/status/" + row["order_id"], headers={'X-Auth': user_token}, verify=True)
            order_status = json.loads(response_status.content)
            download_url = order_status["download_url"]
            if download_url:
                temp_filename = os.path.join(tempfile.gettempdir(), os.path.basename(download_url))
                if not os.path.exists(temp_filename):
                    download_response = requests.get(download_url, stream=True)
                    with open(temp_filename, 'wb') as mesoresfile:
                        shutil.copyfileobj(download_response.raw, mesoresfile)
                    if os.path.exists(temp_filename):
                        print("   Downloaded: ", temp_filename)
                else:
                    print("   Existing: ", temp_filename)
        except KeyError as e:
            print(response_status.content)



